@@ -1,50 +1,71 @@
 /*
  * This file is part of Xpra.
- * Copyright (C) 2016-2020 Antoine Martin <antoine@xpra.org>
+ * Copyright (C) 2016-2021 Antoine Martin <antoine@xpra.org>
  * Copyright (c) 2016 Spikes, Inc.
- * Licensed under MPL 2.1, see:
- * http://www.mozilla.org/MPL/2.1/
+ * Licensed under MPL 2.0, see:
+ * http://www.mozilla.org/MPL/2.0/
  *
  */
 
 'use strict';
 
 const Utilities = {
-	VERSION	: "4.0",
-	REVISION : "0",
-	LOCAL_MODIFICATIONS : "0",
+	VERSION : "4.5.2",
+	REVISION : 1106,
+	LOCAL_MODIFICATIONS : 0,
+	BRANCH : "master",
 
 	exc : function() {
-		console.error.apply(console, arguments);
+		if (console) {
+			console.error.apply(console, arguments);
+		}
 	},
 	error : function() {
-		console.error.apply(console, arguments);
+		if (console) {
+			console.error.apply(console, arguments);
+		}
 	},
 	warn : function() {
-		console.log.apply(console, arguments);
+		if (console) {
+			console.log.apply(console, arguments);
+		}
 	},
 	log : function() {
-		console.log.apply(console, arguments);
+		if (console) {
+			console.log.apply(console, arguments);
+		}
 	},
 	debug : function() {
-		console.debug.apply(console, arguments);
+		if (console) {
+			console.debug.apply(console, arguments);
+		}
 	},
 
 	//these versions should not be redirected:
 	cexc : function() {
-		console.error.apply(console, arguments);
+		if (console) {
+			console.error.apply(console, arguments);
+		}
 	},
 	cerror : function() {
-		console.error.apply(console, arguments);
+		if (console) {
+			console.error.apply(console, arguments);
+		}
 	},
 	cwarn : function() {
-		console.log.apply(console, arguments);
+		if (console) {
+			console.log.apply(console, arguments);
+		}
 	},
 	clog : function() {
-		console.log.apply(console, arguments);
+		if (console) {
+			console.log.apply(console, arguments);
+		}
 	},
 	cdebug : function() {
-		console.debug.apply(console, arguments);
+		if (console) {
+			console.debug.apply(console, arguments);
+		}
 	},
 
 	getHexUUID: function() {
@@ -61,15 +82,18 @@ const Utilities = {
 		return s.join("");
 	},
 
-	getSalt: function(l) {
-		if(l<32 || l>256) {
-			throw 'invalid salt length';
+	getSecureRandomString: function(len) {
+		const crypto = window.crypto || window.mscrypto;
+		if (!crypto) {
+			let s = "";
+			while (s.length<len) {
+				s += Utilities.getHexUUID();
+			}
+			return s.substr(0, len);
 		}
-		let s = '';
-		while (s.length<l) {
-			s += Utilities.getHexUUID();
-		}
-		return s.slice(0, l);
+		const u = new Uint8Array(len);
+		crypto.getRandomValues(u);
+		return String.fromCharCode.apply(null, u);
 	},
 
 	xorString: function(str1, str2){
@@ -151,7 +175,7 @@ const Utilities = {
 		for (let i = 0; i < browserLanguagePropertyKeys.length; i++) {
 			const prop = browserLanguagePropertyKeys[i];
 			language = nav[prop];
-			//console.debug(prop, "=", language);
+			//Utilities.debug(prop, "=", language);
 			if (language && language.length) {
 				return language;
 			}
@@ -161,7 +185,7 @@ const Utilities = {
 
 	getKeyboardLayout: function() {
 		let v = Utilities.getFirstBrowserLanguage();
-		console.debug("getFirstBrowserLanguage()=", v);
+		Utilities.debug("getFirstBrowserLanguage()=", v);
 		if (v==null) {
 			return "us";
 		}
@@ -175,24 +199,15 @@ const Utilities = {
 			}
 			//ie: "en"
 			layout = l[0].toLowerCase();
-			const tmp = LANGUAGE_TO_LAYOUT[v];
+			const tmp = LANGUAGE_TO_LAYOUT[layout];
 			if (tmp) {
 				layout = tmp;
 			}
 		}
-		console.debug("getKeyboardLayout()=", layout);
+		Utilities.debug("getKeyboardLayout()=", layout);
 		return layout;
 	},
 
-	canUseWebP : function() {
-	    const elem = document.createElement('canvas');
-	    const ctx = elem.getContext('2d');
-	    if (!ctx) {
-	    	return false;
-	    }
-        return elem.toDataURL('image/webp').indexOf('data:image/webp') == 0;
-	},
-
 	getAudioContextClass : function() {
 		return window.AudioContext || window.webkitAudioContext || window.audioContext;
 	},
@@ -277,6 +292,10 @@ const Utilities = {
 		return false;
 	},
 
+	isMobile : function () {
+	    return /iPhone|iPad|iPod|Android/i.test(navigator.userAgent);
+	},
+
 	getSimpleUserAgentString : function() {
 		if (Utilities.isFirefox()) {
 			return "Firefox";
@@ -399,13 +418,6 @@ const Utilities = {
 	    }
 	},
 
-	monotonicTime : function() {
-		if (performance) {
-			return Math.round(performance.now());
-		}
-		return Date.now();
-	},
-
 	StringToUint8 : function(str) {
 		const u8a = new Uint8Array(str.length);
 		for(let i=0,j=str.length;i<j;++i){
@@ -423,6 +435,14 @@ const Utilities = {
 		return c.join("");
 	},
 
+	s : function(v){
+		const type = typeof v;
+	    if (type === 'object' && v.constructor===Uint8Array) {
+			return Utilities.Uint8ToString(v);
+		}
+		return v.toString();
+	},
+
 	ArrayBufferToBase64 : function(uintArray) {
 		// apply in chunks of 10400 to avoid call stack overflow
 		// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/apply
@@ -459,7 +479,7 @@ const Utilities = {
 	    const regex = {
 	        section: /^\s*\[\s*([^\]]*)\s*\]\s*$/,
 	        param: /^\s*([^=]+?)\s*=\s*(.*?)\s*$/,
-	        comment: /^\s*;.*$/
+	        comment: /^\s*[;#].*$/
 	    };
 	    const value = {};
 	    const lines = data.split(/[\r\n]+/);
@@ -590,7 +610,49 @@ const Utilities = {
 			i["rtt"] = c.rtt;
 		}
 		return i;
-	}
+	},
+
+	json_action  : function(uri, success_fn, error_fn, username, password) {
+		Utilities.log("json_action(", uri, ", ", success_fn, ", ", error_fn, ")");
+	    var xhr = new XMLHttpRequest();
+	    var url = uri;
+		if (uri.startsWith("/")) {
+			//relative URI
+			url = document.location.href.split("/connect.html")[0] + uri;
+		}
+	    xhr.open('GET', url, true);
+		if (username && password) {
+			xhr.setRequestHeader("Authorization", "Basic " + btoa(username+":"+password));
+		}
+	    xhr.responseType = 'json';
+		xhr.addEventListener("load", function() {
+			Utilities.log("loaded", url, "status", xhr.status);
+			var status = xhr.status;
+			if (status === 200) {
+				success_fn(xhr, xhr.response);
+			}
+			else {
+				Utilities.log(uri, "failed:", status+xhr.response);
+				if (error_fn) {
+					error_fn("failed: "+status+xhr.response);
+				}
+			}
+		});
+		xhr.addEventListener("error", function(e) {
+			Utilities.log(uri, "error:", e);
+			if (error_fn) {
+				error_fn(e);
+			}
+		});
+		xhr.addEventListener("abort", function(e) {
+			Utilities.log(uri, "abort:", e);
+			if (error_fn) {
+				error_fn(e);
+			}
+		});
+		xhr.send();
+		return xhr;
+	},
 };
 
 
